import time, random
from django.core.urlresolvers import resolve
from django.http import HttpRequest
from django.test import TestCase
from ddusers.views import user_list
from ddusers.models import User, Project, WebPage, Publication, DataSet, Country, Affiliation, Institution
from ddusers.views import start_page

from django.utils import translation

from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from selenium.webdriver.chrome.webdriver import WebDriver


def populate_db():
    country_dict = {
        'Armenia': (40.567, 44.620),
        'Azerbaijan': (40.0655055, 48.2022331),
        'Belarus' : (53.6330885, 27.4928052),
        'Georgia' : (42.3029719, 43.1273446),
        'Germany': (1,1),
        'Kazakhstan': (46.6156278, 55.9831199),
        'Kyrgyzstan': (41.1436996, 73.2454583),
        'Moldova': (46.9612588, 28.3),
        'Russia': (57.517, 52.097),
        'Tajikistan': (38.8409856, 69.0037216),
        'Turkmenistan': (38.886136, 55.0833836),
        'Ukraine': (49.3030148, 32.6964561),
        'Uzbekistan': (41.3019792, 62.0803399)
    }

    for key, data in country_dict.items():
        Country.objects.create(name=key, slug=key.lower(), geo_lat=data[0], geo_lng=data[1])


    for x in range(1,12):
        username = 'username_%s' % (x,)
        first_name = 'first_name_%s' % (x,)
        last_name = 'last_name_%s' % (x,)
        middle_name = 'middle_name_%s' % (x,)
        name_prefix = 'Prof. Dr.'
        name_suffix = 'PhD'
        name_of_institution = 'name_of_institution_%s' % (x,)
        place_of_institution = 'place_of_institution_%s' % (x,)
        country_of_institution = 'country_of_institution_%s' % (x,)
        position = 'position_%s' % (x,)

        user = User.objects.create_user(
            username=username, first_name=first_name, last_name=last_name, middle_name=middle_name,
            name_prefix=name_prefix, name_suffix=name_suffix
        )

        country_list = list(country_dict.keys())
        country_name = country_list[random.randrange(0, len(country_list)-1)]
        country = Country.objects.get(name=country_name)
        user.countries.add(country)
        user.save()

        Affiliation.objects.create(user=user, name_of_institution=name_of_institution,
                                   place_of_institution=place_of_institution,
                                   country_of_institution=country_of_institution, position=position
                                   )






class UserPageTest(TestCase):
    def setUp(self):
        webpage = WebPage.objects.create(title='start', main_desc_heading='Startpage', text_left_heading='Left', text_middle_heading='Middle', text_right_heading='Right')
        user1 = User.objects.create_user(username='tuser1', password='pw', first_name='test_user1_first_name',
                                         last_name='test_user1_last_name', middle_name='test_user1_middle_name',
                                         name_prefix='Magister', name_suffix='PhD'
                                         )
        user2 = User.objects.create_user(username='tuser2',  first_name='test2', last_name='user2')
        Project.objects.create(name='project1', user=user1)
        user2.interests.add('interest1')
        user2.interests.add('interest2')
        country1 = Country.objects.create(name='country1')
        country2 = Country.objects.create(name='country2')
        user1.countries.add(country1)
        user1.countries.add(country2)
        institution1 = Institution.objects.create(name='Institution1', address='Adress1', city='City1', country=country1)
        affiliation1 = Affiliation.objects.create(user=user1, name_of_institution='name_of_institution1',
                                                  place_of_institution='place_of_institution1',
                                                  country_of_institution='country_of_institution1', position='position1')
        affiliation2= Affiliation.objects.create(user=user1, name_of_institution='name_of_institution2',
                                                  place_of_institution='place_of_institution2',
                                                  country_of_institution='country_of_institution2',
                                                  position='position2', main=False)
        user1.save()
        populate_db()

    def test_get_academic_name(self):
        user1 = User.objects.get(username='tuser1')
        self.assertEqual(user1.get_academic_name(), 'Magister test_user1_first_name test_user1_middle_name test_user1_last_name, PhD')

    def test_get_main_affiliation(self):
        user1 = User.objects.get(username='tuser1')
        self.assertEqual(user1.get_main_affiliation()[0].name_of_institution, 'name_of_institution1')
        self.assertEqual(user1.get_main_affiliation()[0].position, 'position1')

    def test_get_affiliations(self):
        user1 = User.objects.get(username='tuser1')
        self.assertEqual(user1.get_affiliations()[0].name_of_institution, 'name_of_institution1')
        self.assertEqual(user1.get_affiliations()[1].name_of_institution, 'name_of_institution2')
        self.assertEqual(user1.get_affiliations().count(), 2)

    def test_get_countries(self):
        user1 = User.objects.get(username='tuser1')
        self.assertEqual(user1.get_countries()[0].name, 'country1')
        self.assertEqual(user1.get_countries()[1].name, 'country2')

    def test_get_academic_name(self):
        user1 = User.objects.get(username='tuser1')
        self.assertEqual(user1.get_academic_name(), 'Magister test_user1_first_name test_user1_middle_name test_user1_last_name, PhD')

    def test_get_main_affiliation(self):
        user1 = User.objects.get(username='tuser1')
        self.assertEqual(user1.get_main_affiliation()[0].name_of_institution, 'name_of_institution1')
        self.assertEqual(user1.get_main_affiliation()[0].position, 'position1')

    def test_get_affiliations(self):
        user1 = User.objects.get(username='tuser1')
        self.assertEqual(user1.get_affiliations()[0].name_of_institution, 'name_of_institution1')
        self.assertEqual(user1.get_affiliations()[1].name_of_institution, 'name_of_institution2')
        self.assertEqual(user1.get_affiliations().count(), 2)

    def test_get_countries(self):
        user1 = User.objects.get(username='tuser1')
        self.assertEqual(user1.get_countries()[0].name, 'country1')
        self.assertEqual(user1.get_countries()[1].name, 'country2')

    def test_get_projects(self):
        user1 = User.objects.get(username='tuser1')
        user2 = User.objects.get(username='tuser2')
        self.assertEqual(user1.get_projects()[0].name, 'project1')
        self.assertEqual(user2.get_projects().count(), 0)

    def test_get_interests(self):
        #user1 = User.objects.get(username='test_username1')
        user2 = User.objects.get(username='tuser2')
        self.assertEqual(user2.get_interests().count(), 2)
        #self.assertEqual(user2.get_projects()[0].name, raises(IndexError))

    #def test_user_url_resolves_to_user_listing_view(self):
    #    #translation.activate('en')
    #    found = resolve('/users/user/')
    #    #print('FUNC:', found.func)
    #    self.assertEqual(found.func, user_list)

    def test_user_list_lists_users(self):
        self.client.login(username='tuser1', password='pw')
        response = self.client.get('/users/')
        html = response.content.decode('utf8')
        #self.assertTemplateUsed(response, 'user_list.html')
        #self.assertContains(response, "user1", 1)#
        self.assertContains(response, "last_name_10, PhD", 1)#
        #self.assertContains(response, "user3", 0)#
        self.assertIn('<title>Discuss Data - Researchers</title>', html)

    def test_user_listing_returns_correct_html(self):
        response = self.client.get('/')
        #self.assertTemplateUsed(response, 'index.html')
        request = HttpRequest()
        response = start_page(request)
        html = response.content.decode('utf8')
        self.assertIn('<title>Discuss Data - home</title>', html)



class UserProfileTests(StaticLiveServerTestCase):
    #fixtures = ['user-data.json']

    @classmethod
    def setUpClass(cls):
        super(UserProfileTests, cls).setUpClass()
         # create user
        user = User.objects.create_user(username='fx', first_name='Felix', last_name='Testuser', email=None, password='fx')


        cls.selenium = WebDriver()
        cls.selenium.implicitly_wait(10)

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super().tearDownClass()


    def django_login(self):
        self.client.login(username='fx', password='fx') #Native django test client
        cookie = self.client.cookies['sessionid']
        self.selenium.get(self.live_server_url + '/login/')  #browser will set cookie domain based on current page domain
        self.selenium.add_cookie({'name': 'sessionid', 'value': cookie.value, 'secure': False, 'path': '/'})
        self.selenium.refresh() #need to update page for logged in user
        return self.selenium.get('%s%s' % (self.live_server_url, '/users/user/'))


    def test_user_profile_projects(self):
        #user browses to profile page and checks projects
        self.django_login()

        # add project
        self.selenium.get('%s%s' % (self.live_server_url, '/users/user/edit'))
        self.selenium.find_element_by_id("project_add_button").click()
        self.selenium.find_element_by_id("id_name").send_keys("Test Project")
        self.selenium.find_element_by_id("id_url").click()
        self.selenium.find_element_by_id("id_url").clear()
        self.selenium.find_element_by_id("id_url").send_keys("https://www.test-project.de")
        self.selenium.find_element_by_id("submit-id-save").click()
        bodyText = self.selenium.find_element_by_xpath("//div[@id='user-projects']//h3[.='Test Project']").text
        self.assertTrue("Test Project" in bodyText)

        # check project modal
        self.selenium.get('%s%s' % (self.live_server_url, '/users/user/edit'))
        self.selenium.find_element_by_css_selector('button[class="btn btn-link btn-lg projectEdit"]').click()

        input_name = self.selenium.find_element_by_id('id_name')
        self.assertEqual(
            input_name.get_attribute('value'),
            'Test Project'
        )
        input_name = self.selenium.find_element_by_id('id_url')
        self.assertEqual(
            input_name.get_attribute('value'),
            'https://www.test-project.de'
        )


